const assert = require('assert');
const { Before, Given, When, Then } = require('@cucumber/cucumber');

const axios = require("axios");
const MockAdapter = require("axios-mock-adapter");

// This sets the mock adapter on the default instance
const mockAxios = new MockAdapter(axios);

const { fixtures } = require('../../test/fixtures/server');

const mockValidConfig = fixtures.get('valid-config');
const mockInvalidConfig = fixtures.get('invalid-config');

const mockSchema = require('../../test/mock/swagger.json');
const { Adapter } = require('../../dist/adapter');
const { GenericProtocol } = require('../../dist/protocol');

Before(function() {
  this.mockAxios = mockAxios;
  this.mockAxios.reset();

  this.mockAxios.onGet(
    `${mockValidConfig.api}/token`, 
    { params: { client_id: mockValidConfig.client_id, secret: mockValidConfig.secret } }
  ).reply(200, fixtures.get('response-valid-token'));

  this.mockAxios.onGet(
    `${mockInvalidConfig.api}/token`, 
    { params: { client_id: mockInvalidConfig.client_id, secret: mockInvalidConfig.secret } }
  ).reply(403, fixtures.get('response-denied-token'));

});

Given('valid config', function() {
  this.schema = mockSchema;
  this.config = mockValidConfig
});

When('the adapter instance is created', function() {
  let mockProtocol = new GenericProtocol(this.schema);
  let mockAdapter = new Adapter(mockProtocol, this.config);
  this.adapter = mockAdapter;
});

When('the auth method is called', async function() {
  await this.adapter.auth();
});

Then(('the adapter credentials are populated'), function() { 
  assert.equal(this.adapter.credentials.token, fixtures.get('response-valid-token').token);
});

Given('invalid config', function() {
  this.schema = mockSchema;
  this.config = mockInvalidConfig;
});

Then('the adapter auth fails', async function() {
  this.adapter.auth()
  .catch((error) => {
    assert.equal(error.response.status, 403);
  });
});